(*
 * Copyright 2018 Arne Vajhøj.
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License. 
 *)

[inherit('mysql')]
module pmysql;

type
   byte = [byte] -128..127;
   word = [word] -32768..32767;
   pstr = varying[32000] of char;
   longpstr(n : integer)  = record
                               length : integer;
                               body : packed array[1..n] of char;
                            end;

function stdstr(s : longpstr) : pstr;

begin
   stdstr := substr(s.body, 1, s.length);
end;

procedure pmysql_zero_bind(var bp : mysql_bind);

begin
   bp.length := 0;
   bp.is_null := 0;
   bp.buffer := 0;
   bp.error := 0;
   bp.row_ptr := 0;
   bp.store_param_func := 0;
   bp.fetch_result := 0;
   bp.skip_result := 0;
   bp.buffer_length := 0;
   bp.offset := 0;
   bp.length_value := 0;
   bp.param_number := 0;
   bp.pack_length := 0;
   bp.buffer_type := 0;
   bp.error_value := false;
   bp.is_unsigned := false;
   bp.long_data_used := false;
   bp.is_null_value := false;
   bp.extension := 0;
end;

procedure pmysql_init_bind_tiny(var bp : mysql_bind; var v : byte);

begin
   pmysql_zero_bind(bp);
   bp.buffer_type := MYSQL_TYPE_TINY;
   bp.buffer := iaddress(v);
   bp.buffer_length := 1;
end;

procedure pmysql_init_bind_short(var bp : mysql_bind; var v : word);

begin
   pmysql_zero_bind(bp);
   bp.buffer_type := MYSQL_TYPE_SHORT;
   bp.buffer := iaddress(v);
   bp.buffer_length := 2;
end;

procedure pmysql_init_bind_long(var bp : mysql_bind; var v : integer);

begin
   pmysql_zero_bind(bp);
   bp.buffer_type := MYSQL_TYPE_LONG;
   bp.buffer := iaddress(v);
   bp.buffer_length := 4;
end;

procedure pmysql_init_bind_longlong(var bp : mysql_bind; var v : integer64);

begin
   pmysql_zero_bind(bp);
   bp.buffer_type := MYSQL_TYPE_LONGLONG;
   bp.buffer := iaddress(v);
   bp.buffer_length := 8;
end;

procedure pmysql_init_bind_float(var bp : mysql_bind; var v : real);

begin
   pmysql_zero_bind(bp);
   bp.buffer_type := MYSQL_TYPE_FLOAT;
   bp.buffer := iaddress(v);
   bp.buffer_length := 4;
end;

procedure pmysql_init_bind_double(var bp : mysql_bind; var v : double);

begin
   pmysql_zero_bind(bp);
   bp.buffer_type := MYSQL_TYPE_DOUBLE;
   bp.buffer := iaddress(v);
   bp.buffer_length := 8;
end;

procedure pmysql_init_bind_string_in(var bp : mysql_bind; var v : pstr);

begin
   pmysql_zero_bind(bp);
   bp.buffer_type := MYSQL_TYPE_STRING;
   bp.buffer := iaddress(v.body);
   bp.buffer_length := v.length;
end;

procedure pmysql_init_bind_string_out(var bp : mysql_bind; var v : longpstr);

begin
   pmysql_zero_bind(bp);
   bp.buffer_type := MYSQL_TYPE_STRING;
   bp.buffer := iaddress(v.body);
   bp.buffer_length := upper(v.body);
   bp.length := iaddress(v.length);
end;

function pmysql_init : mysql_ptr;

begin
   crtl_init;
   pmysql_init := mysql_init(0);
end;

function pmysql_real_connect(con : mysql_ptr; host, un, pw, db : pstr; port : integer := 3306; flags : integer := 0) : mysql_ptr;

var
   xhost, xun, xpw, xdb : c_str_t;

begin
   xhost := malloc_c_str(host);
   xun := malloc_c_str(un);
   xpw := malloc_c_str(pw);
   xdb := malloc_c_str(db);
   pmysql_real_connect := mysql_real_connect(con, xhost, xun, xpw, xdb, port, 0, flags);
   free_c_str(xhost);
   free_c_str(xun);
   free_c_str(xpw);
   free_c_str(xdb);
end;

function pmysql_stmt_init(con : mysql_ptr) : mysql_stmt_ptr;

begin
   pmysql_stmt_init := mysql_stmt_init(con);
end;

function pmysql_stmt_prepare(stmt : mysql_stmt_ptr; sql : pstr) : integer;

var
   xsql : c_str_t;
   
begin
   xsql := malloc_c_str(sql);
   pmysql_stmt_prepare := mysql_stmt_prepare(stmt, xsql, sql.length);
   free_c_str(xsql);
end;

function pmysql_stmt_bind_param(stmt : mysql_stmt_ptr; params : array[lb..ub : integer] of mysql_bind) : integer;

begin
   pmysql_stmt_bind_param := mysql_stmt_bind_param(stmt, iaddress(params));
end;

function pmysql_stmt_execute(stmt : mysql_stmt_ptr) : integer;

begin
   pmysql_stmt_execute := mysql_stmt_execute(stmt);
end;

function pmysql_stmt_bind_result(stmt : mysql_stmt_ptr; params : array[lb..ub : integer] of mysql_bind) : integer;

begin
   pmysql_stmt_bind_result := mysql_stmt_bind_result(stmt, iaddress(params));
end;

function pmysql_stmt_store_result(stmt : mysql_stmt_ptr) : integer;

begin
   pmysql_stmt_store_result := mysql_stmt_store_result(stmt);
end;

function pmysql_stmt_fetch(stmt : mysql_stmt_ptr) : integer;

begin
   pmysql_stmt_fetch := mysql_stmt_fetch(stmt);
end;

function pmysql_stmt_free_result(stmt : mysql_stmt_ptr) : integer;

begin
   pmysql_stmt_free_result := mysql_stmt_free_result(stmt);
end;

procedure pmysql_close(con : mysql_ptr);

begin
   mysql_close(con);
end;

function pmysql_error(con : mysql_ptr) : pstr;

begin
   pmysql_error := pas_str(mysql_error(con));
end;

function pmysql_stmt_error(stmt : mysql_stmt_ptr) : pstr;

begin
   pmysql_stmt_error := pas_str(mysql_stmt_error(stmt));
end;

function pmysql_autocommit(con : mysql_ptr; mode : boolean) : integer;

begin
   pmysql_autocommit := mysql_autocommit(con, mode)
end;

function pmysql_commit(con : mysql_ptr) : integer;

begin
   pmysql_commit := mysql_commit(con);
end;

function pmysql_rollback(con : mysql_ptr) : integer;

begin
   pmysql_rollback := mysql_rollback(con);
end;

function pmysql_stmt_affected_rows(stmt : mysql_stmt_ptr) : integer64;

begin
   pmysql_stmt_affected_rows := mysql_stmt_affected_rows(stmt);
end;

function pmysql_stmt_field_count(stmt : mysql_stmt_ptr) : integer;

begin
   pmysql_stmt_field_count := mysql_stmt_field_count(stmt);
end;

function pmysql_stmt_num_rows(stmt : mysql_stmt_ptr) : integer64;

begin
   pmysql_stmt_num_rows := mysql_stmt_num_rows(stmt);
end;

function pmysql_stmt_insert_id(stmt : mysql_stmt_ptr) : integer64;

begin
   pmysql_stmt_insert_id := mysql_stmt_insert_id(stmt);
end;

end.
